<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

use PrestaShop\PrestaShop\Adapter\Presenter\Object\ObjectPresenter;

if (!defined('_PS_VERSION_')) {
    exit;
}

class Etracker extends Module
{
    protected $config_form = false;

    public function isUsingNewTranslationSystem()
    {
        return true;
    }

    public function __construct()
    {
        $this->name = 'etracker';
        $this->tab = 'analytics_stats';
        $this->version = '1.0.0';
        $this->author = 'EVO-CON';
        $this->need_instance = 0;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->trans('etracker', [], 'Modules.Etracker.Etracker');
        $this->description = $this->trans('This module allows you to use both etracker Analytics and Signalize in your shop.', [], 'Modules.Etracker.Etracker');

        $this->confirmUninstall = $this->trans('Are you sure you want to uninstall the etracker module?', [], 'Modules.Etracker.Etracker');

        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue('ETRACKER_LIVE_MODE', false);

        include(dirname(__FILE__).'/sql/install.php');

        file_put_contents(_PS_CORE_DIR_ . '/sw.js', 'importScripts("https://default.signalize.com/sw.js");');

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('actionAdminControllerSetMedia') &&
            $this->registerHook('actionFrontControllerSetMedia') &&
            $this->registerHook('displayHeader');
    }

    public function uninstall()
    {
        Configuration::deleteByName('ETRACKER_LIVE_MODE');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitEtrackerModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $output . $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitEtrackerModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Settings', [], 'Modules.Etracker.Etracker'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-key"></i>',
                        'desc' => $this->trans('Enter your account key', [], 'Modules.Etracker.Etracker'),
                        'name' => 'ETRACKER_SECURE_CODE',
                        'label' => $this->trans('account key', [], 'Modules.Etracker.Etracker'),
                    ),
                ),
                'submit' => array(
                    'title' => $this->trans('Save', [], 'Modules.Etracker.Etracker'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'ETRACKER_LIVE_MODE' => Configuration::get('ETRACKER_LIVE_MODE', true),
            'ETRACKER_SECURE_CODE' => Configuration::get('ETRACKER_SECURE_CODE', null)
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');
    }

    public function hookActionFrontControllerSetMedia($params)
    {
        $this->context->controller->registerStylesheet(
            'module-etracker-style',
            'modules/' . $this->name . '/views/css/front.css',
            [
                'media' => 'all',
                'priority' => 200,
            ]
        );

        $this->context->controller->registerJavascript(
            'module-etracker-simple-lib',
            'modules/' . $this->name . '/views/js/front.js',
            [
                'priority' => 200,
                'attribute' => 'async',
            ]
        );
    }

    public function hookActionAdminControllerSetMedia($params)
    {
        $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        $this->context->controller->addJS($this->_path . 'views/js/back.js');
    }

    public function hookDisplayHeader()
    {
        $this->smarty->assign([
            'secure_code' => Configuration::get('ETRACKER_SECURE_CODE', null),
            'page' => $this->context->controller->php_self,
            'cart_link' => $this->context->link->getPageLink('cart'),
            'ajax_url' => $this->context->link->getModuleLink('etracker', 'ajax')
        ]);

        if ($this->context->controller->php_self == 'product') {
            $objProduct = $this->context->controller->getProduct();
            $objPresenter = new ObjectPresenter();
            $arrProduct = $objPresenter->present($objProduct);
            $arrProduct['id_product'] = (int) $arrProduct['id'];
            $arrProduct['out_of_stock'] = (int) $objProduct->out_of_stock;
            $product_full = Product::getProductProperties($this->context->language->id, $arrProduct, $this->context);

            $this->smarty->assign([
                'arrProduct' => $product_full,
                'objCurrency' => $this->context->currency
            ]);
        }
        if ($this->context->controller->php_self == 'order-confirmation') {
            $objOrder = new Order($this->context->controller->id_order);
            $objCart = new Cart($objOrder->id_cart);

            $objBasket = ['id' => 'cart_' . $objCart->id, 'products' => []];

            $arrProducts = $objCart->getProducts();

            foreach ($arrProducts as $arrProduct) {
                $objProduct = new Product($arrProduct['id_product']);
                $arrCategoryIDs = $objProduct->getCategories();
                foreach ($arrCategoryIDs as $intCategoryID) {
                    $objCategory = new Category($intCategoryID);
                    if ($objCategory->is_root_category === '0') {
                        continue;
                    }
                }

                $objBasket['products'][] = [
                    'product' => [
                        'id' => $arrProduct['id_product'],
                        'name' => $arrProduct['name'],
                        'price' => (string)$arrProduct['price'],
                        'currency' => $this->context->currency->iso_code
                    ],
                    'quantity' => (int)$arrProduct['cart_quantity']
                ];
            }

            $this->smarty->assign([
                'objOrder' => $objOrder,
                'jsonBasket' => json_encode($objBasket),
                'objCurrency' => $this->context->currency
            ]);
        }

        return $this->display(__FILE__, 'views/templates/hook/header.tpl');
    }
}
